/**
  This example is used to see the wind conditions of a location from 7 to 19 of the current day, up to the next 3 days.
  It connects to an NTP server to set today's date / time and takes daylight saving time into account if you set the TimeZone string correctly.
  Look at  https://github.com/nayarsystems/posix_tz_db/blob/master/zones.csv  for set the correct string for your TimeZone
  Fills a table with data arriving from the server https://www.windfinder.com (to which go my thanks for the service provided),
  and warns if the minimum wind detected exceeds a certain threshold that you can establish.
  The warning starts only if it is the central hours of the day (excluding 7 and 19), if the hour is after the current one and if the cloud cover is below 100%).
  You can implement a Send Email if you want, or start a music when event raise
  Warnig stop to flash on screen if you click on em.
  You can select the language in which the dates are displayed in the table (among those supported by the WindFinder server).
  Click on the LED at the top right for a software reset.
  The preferences are saved on SPIFFS if you change the minimum wind, or if you click on the date of the day or if you change language.
  When the day changes, the program reconnects to see if the weather conditions have changed.

  in Appendix you can see how strftime work

   Created by G.Ponte
   Email: gabrielepippo034@gmail.com
   Copyright (c) 2022 G.Ponte
*/
#include <lvgl.h>
#include "FS.h"
#include <TFT_eSPI.h>
#include "time.h"
#include <WiFi.h>
#include <WiFiClient.h>
#include <HTTPClient.h>
#include "SPIFFS.h"
TFT_eSPI tft = TFT_eSPI(); /* TFT instance */
#include <SimpleFTPServer.h>/* https://github.com/xreef/SimpleFTPServer      download LAST version*/
const int screenWidth = 480;
const int  screenHeight = 320;
static lv_color_t buf[screenWidth * screenHeight / 10];
static lv_disp_buf_t disp_buf;
static lv_obj_t * screen;
static lv_style_t StyleBTN;
static lv_style_t styleTavola1_bg;
static lv_style_t styleTavola1_cell;
static lv_style_t StyleLed;
static lv_obj_t * Led1 ;
static lv_obj_t * Avviso ;
static lv_obj_t * Tavola1 ;
static lv_obj_t * Strip1;
static lv_obj_t * Strip2;
static lv_obj_t * Strip3;
static lv_obj_t * Strip4;
static lv_obj_t *Alert;
static lv_obj_t *Orologio1;
static lv_obj_t *Orologio2;
static lv_obj_t *Dropdown1;
static lv_obj_t *Dropdown2;
static lv_obj_t *Dropdown3;
static lv_obj_t *Sunrise1;
static lv_obj_t *Sunrise2;
static lv_obj_t *Lat1;
static lv_obj_t *Lat2;
File pFileOut;
char TT[512];
String T = "";
String ssid = "YOUR WiFI SSID";
String wifipw = "YOUR WiFi Password";
String Language = "it";
int LanguagePos = 0;
unsigned long previousMillis = 0;
unsigned long interval = 5000;
unsigned long AlertInterval = millis();
unsigned long UpdateClock = millis();
String DataOggi = "";
String DataOld = "";
String CurrentHour = "";
String CittaSelezionata = "milano_city";
int CittaSelezionataPos = 0;
int MinWind = 10;
int MinWindPos = 0;
int StartStatus = 0;
int AvvisoStart = 1;
int PreferencesFound = 0;
FtpServer ftpSrv;
time_t now;
char strftime_buf[64];
struct tm timeinfo;
String TimeZone = "CET-1CEST,M3.5.0,M10.5.0/3"; /* Rome https://github.com/nayarsystems/posix_tz_db/blob/master/zones.csv */
String Richiesta1 = "https://www.windfinder.com/widget/forecast/";
String Richiesta2 = "?days=4&columns=1&unit_wind=kmh&unit_temperature=c&unit_wave=m&unit_rain=mm&unit_pressure=mbar&label_wind=km%2Fh&show_wind=1&show_temperature=0&show_waves=0&show_rain=0&show_clouds=1&show_pressure=0&show_day=1&version=1";
String Richiesta3 = Richiesta1 + CittaSelezionata + Richiesta2;
String Risult[4] ;/* 0 città   1 giorno    2 ora     3 velocità   4 sole-pioggia*/
int InvioMail  = 0;
int AggiungoTempo  = 0;
int Trovo1; int Trovo2; int Trovo3;
/* HANDLERS START      HANDLERS START      HANDLERS START  */
/* ======     Display flushing      Display flushing    ========*/
void my_disp_flush(lv_disp_drv_t *disp, const lv_area_t *area, lv_color_t *color_p) {
  uint16_t c;
  tft.startWrite(); /* Start new TFT transaction */
  tft.setAddrWindow(area->x1, area->y1, (area->x2 - area->x1 + 1), (area->y2 - area->y1 + 1)); /* set the working window */
  for (int y = area->y1; y <= area->y2; y++) {
    for (int x = area->x1; x <= area->x2; x++) {
      c = color_p->full; tft.writeColor(c, 1); color_p++;
    }
  }
  tft.endWrite(); /* terminate TFT transaction */
  lv_disp_flush_ready(disp); /* tell lvgl that flushing is done */
}
/* ===========   READ TOUCHPAD ============ */
bool my_input_read(lv_indev_drv_t * indev_driver, lv_indev_data_t * data) {
  uint16_t touchX, touchY;
  bool touched = tft.getTouch(&touchX, &touchY, 600);
  if (!touched) {
    return false;
  }
  if (touchX > screenWidth || touchY > screenHeight) {
  } else {
    data->state = touched ? LV_INDEV_STATE_PR : LV_INDEV_STATE_REL;
    data->point.x = touchX;
    data->point.y = touchY;
  }
  return false; /*Return `false` because we are not buffering and no more data to read*/
}
static void Orologio2_eventhandler(lv_obj_t * Orologio2, lv_event_t event) {
  if (event == LV_EVENT_PRESSED) {
    SalvoPref( );
  }
}
static void Avviso_eventhandler(lv_obj_t * Avviso, lv_event_t event) {/*     LABEL TOAST INFO EVENT    */
  if (event == LV_EVENT_PRESSED) {
    AvvisoStart = 0; lv_obj_set_hidden(Avviso, true);
  }
}
static void Led1_eventhandler(lv_obj_t * Led1, lv_event_t event) {
  if (event == LV_EVENT_PRESSED) {
    ESP.restart();/* RESET*/
  }
}
static void Dropdown1_eventhandler(lv_obj_t * obj, lv_event_t event) {
  if (event == LV_EVENT_VALUE_CHANGED) {
    lv_dropdown_get_selected_str(obj, TT, sizeof(TT));
    lv_obj_align(Dropdown1, screen, LV_ALIGN_IN_TOP_LEFT, 10, 70);
    Richiesta3 = Richiesta1 + String(TT) + Richiesta2;
    CittaSelezionata = String(TT);
    CittaSelezionataPos = lv_dropdown_get_selected(obj);
    lv_label_set_text(Alert, "");
    LeggoWeb();
  }
}
static void Dropdown2_eventhandler(lv_obj_t * obj, lv_event_t event) {
  if (event == LV_EVENT_VALUE_CHANGED) {
    lv_dropdown_get_selected_str(obj, TT, sizeof(TT));
    T = String(TT); MinWind = T.toInt();
    MinWindPos = lv_dropdown_get_selected(obj);
    T = "Minimum Wind Alert set to " + (String)MinWind;
    TT[(T.length() + 1)]; T.toCharArray(TT, T.length() + 1);
    lv_label_set_text(Alert, TT); lv_obj_align(Alert, screen, LV_ALIGN_IN_TOP_MID, 0, 13);
    lv_obj_set_hidden(Alert, false);
    AlertInterval =  millis();
    SalvoPref( );
  }
}
static void Dropdown3_eventhandler(lv_obj_t * obj, lv_event_t event) {
  if (event == LV_EVENT_VALUE_CHANGED) {
    lv_dropdown_get_selected_str(obj, TT, sizeof(TT));
    T = String(TT); Language = T;
    LanguagePos = lv_dropdown_get_selected(obj);
    T = "Language Select is " + Language;
    TT[(T.length() + 1)]; T.toCharArray(TT, T.length() + 1);
    lv_label_set_text(Alert, TT); lv_obj_align(Alert, screen, LV_ALIGN_IN_TOP_MID, 0, 13);
    lv_obj_set_hidden(Alert, false);
    AlertInterval =  millis();
    SalvoPref( );
  }
}
/* ======    SETUP     SETUP      SETUP       SETUP  ===== */
void setup() {
  WiFi.disconnect(true);
  Serial.begin(115200); /* prepare for possible serial debug */
  delay(100);
 /* if (SPIFFS.begin(true)) {   set TRUE if want format SPIFFS if mount fail */
    if (SPIFFS.begin()) {
    Serial.println(F("SPIFFS mounted correctly."));
    pFileOut = SPIFFS.open("/test.txt", "r");
  } else {
    Serial.println(F("!An error occurred during SPIFFS mounting"));
    Serial.println(F("!Probably you need format SPIFFS"));
    Serial.println(F("!We RESET now"));
    ESP.restart();/* RESET*/
  }
  lv_init();
  tft.begin(); /* TFT init */
  tft.setRotation(3);
  uint16_t calData[5] = { 275, 3620, 264, 3532, 1 };
  tft.setTouch(calData);
  lv_disp_buf_init(&disp_buf, buf, NULL, screenWidth * screenHeight / 10);
  /* ==== Initialize the DISPLAY ====*/
  lv_disp_drv_t disp_drv;
  lv_disp_drv_init(&disp_drv);
  disp_drv.hor_res = screenWidth;
  disp_drv.ver_res = screenHeight;
  disp_drv.flush_cb = my_disp_flush;
  disp_drv.buffer = &disp_buf;
  lv_disp_drv_register(&disp_drv);
  /* ==== Initialize the INPUT DEVICE DRIVER ====*/
  lv_indev_drv_t indev_drv;
  lv_indev_drv_init(&indev_drv);      /*Basic initialization*/
  indev_drv.type = LV_INDEV_TYPE_POINTER;   /*Touch pad is a pointer-like device*/
  indev_drv.read_cb = my_input_read;
  /*Register the driver in LVGL and save the created input device object*/
  lv_indev_drv_register(&indev_drv);

  /* ==== GUI Objects ==== */
  lv_style_set_bg_color(&StyleBTN,   LV_STATE_DEFAULT, lv_color_hex(106 * 256 * 256 + 151 * 256 + 196));
  lv_style_set_text_color(&StyleBTN,  LV_STATE_DEFAULT, lv_color_hex(255 * 256 * 256 + 255 * 256 + 255));
  lv_style_set_text_opa(&StyleBTN, LV_STATE_DEFAULT, 255);
  lv_style_set_text_font(&StyleBTN, LV_STATE_DEFAULT, &lv_font_montserrat_20);
  lv_style_set_bg_opa(&StyleBTN, LV_STATE_DEFAULT, 255);
  lv_style_set_bg_grad_color(&StyleBTN, LV_STATE_DEFAULT, lv_color_hex(25 * 256 * 256 + 69 * 256 + 113));
  lv_style_set_bg_main_stop(&StyleBTN, LV_STATE_DEFAULT, 100);
  lv_style_set_bg_grad_stop(&StyleBTN, LV_STATE_DEFAULT, 160);
  lv_style_set_radius(&StyleBTN, LV_STATE_DEFAULT, 12);
  lv_style_set_bg_grad_dir(&StyleBTN, LV_STATE_DEFAULT, LV_GRAD_DIR_VER);
  lv_style_set_border_color(&StyleBTN, LV_STATE_DEFAULT, lv_color_hex(181 * 256 * 256 + 218 * 256 + 255));
  lv_style_set_border_opa(&StyleBTN, LV_STATE_DEFAULT, 255);
  lv_style_set_border_width(&StyleBTN, LV_STATE_DEFAULT, 1);
  lv_style_set_border_side(&StyleBTN, LV_STATE_DEFAULT, LV_BORDER_SIDE_FULL);
  lv_style_set_shadow_color(&StyleBTN, LV_STATE_DEFAULT, lv_color_hex(156 * 256 * 256 + 184 * 256 + 211));
  lv_style_set_shadow_opa(&StyleBTN, LV_STATE_DEFAULT, 200);
  lv_style_set_shadow_width(&StyleBTN, LV_STATE_DEFAULT, 30);
  lv_style_set_shadow_spread(&StyleBTN, LV_STATE_DEFAULT, 5);
  lv_style_set_pad_left(&StyleBTN, LV_STATE_DEFAULT, 8);
  lv_style_set_pad_right(&StyleBTN, LV_STATE_DEFAULT, 8);
  lv_style_set_pad_top(&StyleBTN, LV_STATE_DEFAULT, 2);
  lv_style_set_pad_bottom(&StyleBTN, LV_STATE_DEFAULT, 2);

  screen = lv_obj_create(NULL, NULL);
  lv_disp_load_scr( screen ) ;/*  <<<<<<<<<<  IMPORTANT IMPORTANT IMPORTANT IMPORTANT IMPORTANT IMPORTANT ------------*/
  lv_obj_set_style_local_bg_color(screen, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, lv_color_hex(0 * 256 * 256 + 0 * 256 + 0 ));
  lv_obj_set_style_local_bg_opa(screen, LV_OBJ_PART_MAIN, LV_STATE_DEFAULT, 255);

  Led1 = lv_led_create(screen, NULL);
  lv_obj_set_click(Led1, true);
  lv_style_reset(&StyleLed);
  lv_style_set_radius(&StyleLed,  LV_STATE_DEFAULT, 8);
  lv_style_set_bg_color(&StyleLed, LV_STATE_DEFAULT, lv_color_hex(200 * 256 * 256 + 0 * 256 + 0 ));
  lv_style_set_bg_opa(&StyleLed,  LV_STATE_DEFAULT, 255);
  lv_style_set_shadow_color(&StyleLed, LV_STATE_DEFAULT, lv_color_hex(200 * 256 * 256 + 0 * 256 + 0 ));
  lv_style_set_shadow_width(&StyleLed, LV_STATE_DEFAULT, 20);
  lv_style_set_shadow_opa(&StyleLed, LV_STATE_DEFAULT, 240);
  lv_style_set_border_color(&StyleLed, LV_STATE_DEFAULT, lv_color_hex(255 * 256 * 256 + 255 * 256 + 255 ));
  lv_style_set_border_width(&StyleLed,  LV_STATE_DEFAULT, 2);
  lv_style_set_border_opa(&StyleLed,  LV_STATE_DEFAULT, 100);
  lv_obj_add_style(Led1 ,  LV_LED_PART_MAIN, &StyleLed );
  lv_obj_set_pos(Led1, 430, 20);
  lv_obj_set_size(Led1, 25, 10);
  lv_led_set_bright(Led1, 200);
  lv_obj_set_event_cb(Led1, Led1_eventhandler);/*Assign an event function*/

  Tavola1 = lv_table_create(screen, NULL);
  lv_obj_set_click(Tavola1, false);
  lv_obj_clear_state(Tavola1, LV_STATE_DISABLED);
  lv_obj_set_drag(Tavola1, false);
  lv_obj_set_hidden(Tavola1, false);
  /*Write style LV_TABLE_PART_BG for Tavola1*/
  lv_style_reset(&styleTavola1_bg);
  lv_style_set_bg_color(&styleTavola1_bg, LV_STATE_DEFAULT, lv_color_hex(55 * 256 * 256 + 90 * 256 + 171 ));
  lv_style_set_bg_grad_color(&styleTavola1_bg, LV_STATE_DEFAULT, lv_color_hex(78 * 256 * 256 + 123 * 256 + 229 ));
  lv_obj_set_style_local_bg_main_stop(Tavola1, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, 118);
  lv_obj_set_style_local_bg_grad_stop(Tavola1, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, 130);
  lv_obj_set_style_local_radius(Tavola1, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, 10);
  lv_style_set_bg_grad_dir(&styleTavola1_bg, LV_STATE_DEFAULT, LV_GRAD_DIR_HOR);
  lv_style_set_bg_opa(&styleTavola1_bg, LV_STATE_DEFAULT, 255);
  lv_style_set_border_color(&styleTavola1_bg, LV_STATE_DEFAULT, lv_color_hex(53 * 256 * 256 + 125 * 256 + 125 ));
  lv_style_set_border_width(&styleTavola1_bg, LV_STATE_DEFAULT, 1);
  lv_style_set_text_color(&styleTavola1_bg, LV_STATE_DEFAULT, lv_color_hex(232 * 256 * 256 + 232 * 256 + 240 ));
  lv_style_set_text_decor(&styleTavola1_bg , LV_STATE_DEFAULT , LV_TEXT_DECOR_NONE);
  lv_style_set_text_font(&styleTavola1_bg, LV_STATE_DEFAULT, &lv_font_montserrat_12);
  lv_style_set_pad_left(&styleTavola1_bg,  LV_STATE_DEFAULT, 0);
  lv_style_set_pad_right(&styleTavola1_bg, LV_STATE_DEFAULT, 0);
  lv_style_set_pad_top(&styleTavola1_bg, LV_STATE_DEFAULT, 0);
  lv_style_set_pad_bottom(&styleTavola1_bg, LV_STATE_DEFAULT, 0);
  lv_obj_add_style(Tavola1, LV_OBJ_PART_MAIN, &styleTavola1_bg);
  lv_style_reset(&styleTavola1_cell);
  lv_style_set_border_color(&styleTavola1_cell, LV_STATE_DEFAULT, lv_color_hex(39 * 256 * 256 + 39 * 256 + 95 ));
  lv_style_set_border_width(&styleTavola1_cell, LV_STATE_DEFAULT, 1);
  lv_style_set_pad_left(&styleTavola1_cell,  LV_STATE_DEFAULT, 0);
  lv_style_set_pad_right(&styleTavola1_cell, LV_STATE_DEFAULT, 0);
  lv_style_set_pad_top(&styleTavola1_cell, LV_STATE_DEFAULT, 0);
  lv_style_set_pad_bottom(&styleTavola1_cell, LV_STATE_DEFAULT, 0);
  lv_obj_add_style(Tavola1 , LV_TABLE_PART_CELL1, &styleTavola1_cell);
  lv_obj_set_style_local_radius(Tavola1, LV_TABLE_PART_CELL1, LV_STATE_DEFAULT, 10);
  lv_table_set_col_cnt(Tavola1, 6);
  lv_table_set_row_cnt(Tavola1, 16);
  lv_obj_set_size(Tavola1, 350, ((lv_table_get_row_cnt(Tavola1) + 1) * 32));
  lv_table_set_col_width(Tavola1, 0,  116);
  lv_table_set_col_width(Tavola1, 1, 24); lv_table_set_col_width(Tavola1, 2, 24); lv_table_set_col_width(Tavola1, 3, 24);
  lv_table_set_col_width(Tavola1, 4, 24); lv_table_set_col_width(Tavola1, 5, 24);

  for (int Colon = 0; Colon < lv_table_get_col_cnt(Tavola1)  ; Colon++) {
    for (int Riga = 0; Riga < lv_table_get_row_cnt(Tavola1)  ; Riga++) {
      if (Colon == 0 ) {
        for (int i = 0; i < 4 ; i++) {
          if (Riga % 4 == 0) {/*se non c'è resto nella divisione*/
            switch (i) {
              case (0) : { /* ---------- Date-------- */
                  T = "Saturday,Feb11";
                  break;
                }
              case (1) : { /* ---------- Min wind-------- */
                  T = "Wind Min";
                  break;
                }
              case (2) : { /* ---------- Max wind------- */
                  T = "Wind Max";
                  break;
                }
              case (3) : { /* ---------- Date-------- */
                  T = "Clouds";
                  break;
                }
              default:
                break;
            }
            TT[(T.length() + 1)]; T.toCharArray(TT, T.length() + 1);
            lv_table_set_cell_value(Tavola1, Riga + i, 0, TT);
            /*Serial.println("Riga=" + (String)int(Riga) + "   I=" + (String)i + "    " +(String)(Riga % 4)+"  "+ TT);*/
          }
        }
      } else {
        if (Riga % 4 == 0) {/*se non c'è resto nella divisione*/
          switch (Colon) {
            case (1) : { /* ---------- Hour 7-------- */
                T = "7";
                break;
              }
            case (2) : { /* ---------- Hour 10 ------- */
                T = "10";
                break;
              }
            case (3) : { /* ---------- Hour 13------- */
                T = "13";
                break;
              }
            case (4) : { /* ---------- Hour 16-------- */
                T = "16";
                break;
              }
            case (5) : { /* ---------- Hour 19-------- */
                T = "19";
                break;
              }
            default:
              break;
          }
          TT[(T.length() + 1)]; T.toCharArray(TT, T.length() + 1);
          lv_table_set_cell_value(Tavola1, Riga, Colon, TT);
        } else {
          lv_table_set_cell_value(Tavola1, Riga, Colon, "0");
        }
      }
      lv_table_set_cell_align(Tavola1, Riga, Colon, LV_LABEL_ALIGN_CENTER);
    }
  }
  lv_obj_align(Tavola1, screen, LV_ALIGN_IN_TOP_MID, 115, 65);

  Strip1 = lv_label_create(Tavola1, NULL);/*-------------------------- LABEL STRIP1 ----------------------------*/
  lv_label_set_long_mode(Strip1, LV_LABEL_LONG_CROP);
  lv_label_set_align(Strip1, LV_LABEL_ALIGN_CENTER);
  /*lv_label_set_text(Strip1,  LV_SYMBOL_SETTINGS " PREF");*/
  lv_obj_set_size(Strip1, 226, 13);
  lv_obj_set_click(Strip1, false);
  lv_obj_set_hidden(Strip1, false);
  lv_obj_set_drag(Strip1, false);
  lv_label_set_text(Strip1, "");
  lv_obj_align(Strip1, Tavola1,  LV_ALIGN_IN_TOP_MID, 0, 1);
  lv_obj_set_style_local_bg_color(Strip1, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, lv_color_hex(255 * 256 * 256 + 255 * 256 + 20));
  lv_obj_set_style_local_bg_opa(Strip1, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, 50);
  lv_obj_set_style_local_radius(Strip1, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, 10);
  Strip2 = lv_label_create(Tavola1, Strip1);/*-------------------------- LABEL STRIP2 ----------------------------*/
  lv_obj_align(Strip2, Tavola1,  LV_ALIGN_IN_TOP_MID, 0, 61);
  Strip3 = lv_label_create(Tavola1, Strip1);/*-------------------------- LABEL STRIP3 ----------------------------*/
  lv_obj_align(Strip3, Tavola1,  LV_ALIGN_IN_TOP_MID, 0, 121);
  Strip4 = lv_label_create(Tavola1, Strip1);/*-------------------------- LABEL STRIP4 ----------------------------*/
  lv_obj_align(Strip4, Tavola1,  LV_ALIGN_IN_TOP_MID, 0, 181);

  Orologio1 = lv_label_create(screen, NULL);
  lv_label_set_long_mode(Orologio1, LV_LABEL_LONG_CROP);
  lv_label_set_align(Orologio1, LV_LABEL_ALIGN_CENTER);
  lv_label_set_text(Orologio1, "00:00:00");
  lv_obj_set_size(Orologio1, 120, 28);
  lv_obj_set_click(Orologio1, false);
  lv_obj_set_hidden(Orologio1, true);
  lv_obj_set_drag(Orologio1, false);
  lv_obj_add_style(Orologio1, LV_OBJ_PART_MAIN, &StyleBTN);
  lv_obj_align(Orologio1, screen, LV_ALIGN_IN_TOP_MID, -80, 10);

  Orologio2 = lv_label_create(screen, NULL);
  lv_label_set_long_mode(Orologio2,  LV_LABEL_LONG_CROP );
  lv_label_set_align(Orologio2, LV_LABEL_ALIGN_CENTER);
  lv_label_set_text(Orologio2, "00:00:00");
  lv_obj_set_size(Orologio2, 130, 28);
  lv_obj_set_click(Orologio2, true);
  lv_obj_set_hidden(Orologio2, true);
  lv_obj_clear_state(Orologio2, LV_STATE_DISABLED);
  lv_obj_set_drag(Orologio2, false);
  lv_obj_add_style(Orologio2, LV_OBJ_PART_MAIN, &StyleBTN);
  lv_obj_align(Orologio2, screen, LV_ALIGN_IN_TOP_MID, 75, 10);
  lv_obj_set_event_cb(Orologio2, Orologio2_eventhandler);/*Assign an event function*/

  Alert = lv_label_create(screen, NULL);/* ALERT    ALERT    ALERT    ALERT    ALERT    ALERT    ALERT    */
  lv_label_set_long_mode(Alert, LV_LABEL_LONG_EXPAND);
  lv_label_set_align(Alert, LV_LABEL_ALIGN_CENTER);
  lv_label_set_text(Alert, "WiFi connection in 5 seconds ");
  lv_obj_set_size(Alert, 81, 16);
  lv_obj_set_click(Alert, false);
  lv_obj_set_hidden(Alert, false);
  lv_obj_clear_state(Alert, LV_STATE_DISABLED);
  lv_obj_set_drag(Alert, false);
  lv_obj_set_style_local_text_color(Alert, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, lv_color_hex(226 * 256 * 256 + 226 * 256 + 226));
  lv_obj_set_style_local_text_opa(Alert, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, 255);
  lv_obj_set_style_local_text_font(Alert, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, &lv_font_montserrat_12);
  lv_obj_set_style_local_bg_color(Alert, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, lv_color_hex(43 * 256 * 256 + 100 * 256 + 157));
  lv_obj_set_style_local_bg_opa(Alert, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, 255);
  lv_obj_set_style_local_bg_main_stop(Alert, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, 0);
  lv_obj_set_style_local_bg_grad_stop(Alert, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, 255);
  lv_obj_set_style_local_radius(Alert, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, 12);
  lv_obj_set_style_local_border_color(Alert, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, lv_color_hex(181 * 256 * 256 + 218 * 256 + 255));
  lv_obj_set_style_local_border_opa(Alert, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, 255);
  lv_obj_set_style_local_border_width(Alert, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, 1);  lv_obj_set_style_local_shadow_color(Alert, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, lv_color_hex(0 * 256 * 256 + 0 * 256 + 0));
  lv_obj_set_style_local_shadow_opa(Alert, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, 240);
  lv_obj_set_style_local_shadow_width(Alert, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, 60);
  lv_obj_set_style_local_shadow_spread(Alert, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, 16);
  lv_obj_set_style_local_pad_left(Alert, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, 8);
  lv_obj_set_style_local_pad_right(Alert, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, 8);
  lv_obj_set_style_local_pad_top(Alert, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, 5);
  lv_obj_set_style_local_pad_bottom(Alert, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, 5);
  lv_obj_align(Alert, screen, LV_ALIGN_IN_TOP_MID, 0, 13);

  Avviso = lv_label_create(screen, Alert);
  lv_obj_add_style(Avviso, LV_OBJ_PART_MAIN, &StyleBTN);
  lv_obj_set_style_local_text_font(Avviso, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, &lv_font_montserrat_14);
  lv_obj_set_size(Avviso, 200, 17);
  lv_label_set_text(Avviso, "");
  lv_obj_set_click(Avviso, true);
  lv_obj_set_hidden(Avviso, true);
  lv_obj_set_drag(Avviso, false);
  lv_obj_align(Avviso, screen,  LV_ALIGN_CENTER, 0, 110);
  lv_obj_set_event_cb(Avviso, Avviso_eventhandler);

  Dropdown1 = lv_dropdown_create(screen, NULL);
  lv_dropdown_set_dir( Dropdown1, LV_DROPDOWN_DIR_DOWN);
  lv_dropdown_set_options( Dropdown1, "tolcinasco_milano\n"
                           "milano-linate\n"
                           "milano_city\n"
                           "molo\n"
                           "dormelletto\n"
                           "lecco_galbiate");
  lv_dropdown_set_text( Dropdown1, "Select City");
  lv_dropdown_set_max_height( Dropdown1, 260);
  lv_obj_set_size(Dropdown1, 160, 24);
  lv_obj_set_click(Dropdown1, true);
  lv_obj_set_hidden(Dropdown1, true);
  lv_obj_clear_state(Dropdown1, LV_STATE_DISABLED);
  lv_obj_set_drag(Dropdown1, false);
  lv_obj_align(Dropdown1, screen, LV_ALIGN_IN_TOP_LEFT, 10, 70);
  lv_dropdown_set_show_selected(Dropdown1, true);
  lv_obj_set_event_cb(Dropdown1, Dropdown1_eventhandler);
  lv_obj_set_style_local_text_color(Dropdown1, LV_DROPDOWN_PART_MAIN, LV_STATE_DEFAULT, lv_color_hex(214 * 256 * 256 + 231 * 256 + 249));
  lv_obj_set_style_local_text_opa(Dropdown1, LV_DROPDOWN_PART_MAIN, LV_STATE_DEFAULT, 255);
  lv_obj_set_style_local_text_font(Dropdown1, LV_DROPDOWN_PART_MAIN, LV_STATE_DEFAULT, &lv_font_montserrat_14);
  lv_obj_set_style_local_bg_color(Dropdown1, LV_DROPDOWN_PART_MAIN, LV_STATE_DEFAULT, lv_color_hex(25 * 256 * 256 + 69 * 256 + 113));
  lv_obj_set_style_local_bg_opa(Dropdown1, LV_DROPDOWN_PART_MAIN, LV_STATE_DEFAULT, 255);
  lv_obj_set_style_local_border_color(Dropdown1, LV_DROPDOWN_PART_MAIN, LV_STATE_DEFAULT, lv_color_hex(181 * 256 * 256 + 218 * 256 + 255));
  lv_obj_set_style_local_border_opa(Dropdown1, LV_DROPDOWN_PART_MAIN, LV_STATE_DEFAULT, 255);
  lv_obj_set_style_local_border_width(Dropdown1, LV_DROPDOWN_PART_MAIN, LV_STATE_DEFAULT, 1);
  lv_obj_set_style_local_shadow_color(Dropdown1, LV_DROPDOWN_PART_MAIN, LV_STATE_DEFAULT, lv_color_hex(187 * 256 * 256 + 207 * 256 + 252));
  lv_obj_set_style_local_shadow_opa(Dropdown1, LV_DROPDOWN_PART_MAIN, LV_STATE_DEFAULT, 200);
  lv_obj_set_style_local_shadow_width(Dropdown1, LV_DROPDOWN_PART_MAIN, LV_STATE_DEFAULT, 30);
  lv_obj_set_style_local_shadow_spread(Dropdown1, LV_DROPDOWN_PART_MAIN, LV_STATE_DEFAULT, 5);
  lv_obj_set_style_local_pad_left(Dropdown1, LV_DROPDOWN_PART_MAIN, LV_STATE_DEFAULT, 5);
  lv_obj_set_style_local_pad_right(Dropdown1, LV_DROPDOWN_PART_MAIN, LV_STATE_DEFAULT, 5);
  lv_obj_set_style_local_pad_top(Dropdown1, LV_DROPDOWN_PART_MAIN, LV_STATE_DEFAULT, 2);
  lv_obj_set_style_local_pad_bottom(Dropdown1, LV_DROPDOWN_PART_MAIN, LV_STATE_DEFAULT, 2);
  lv_obj_set_style_local_text_color(Dropdown1, LV_DROPDOWN_PART_LIST, LV_STATE_DEFAULT, lv_color_hex(187 * 256 * 256 + 207 * 256 + 252));
  lv_obj_set_style_local_text_opa(Dropdown1, LV_DROPDOWN_PART_LIST, LV_STATE_DEFAULT, 255);
  lv_obj_set_style_local_text_font(Dropdown1, LV_DROPDOWN_PART_LIST, LV_STATE_DEFAULT, &lv_font_montserrat_14);
  lv_obj_set_style_local_bg_color(Dropdown1, LV_DROPDOWN_PART_LIST, LV_STATE_DEFAULT, lv_color_hex(55 * 256 * 256 + 90 * 256 + 171));
  lv_obj_set_style_local_bg_opa(Dropdown1, LV_DROPDOWN_PART_LIST, LV_STATE_DEFAULT, 255);
  lv_obj_set_style_local_bg_main_stop(Dropdown1, LV_DROPDOWN_PART_LIST, LV_STATE_DEFAULT, 0);
  lv_obj_set_style_local_bg_grad_stop(Dropdown1, LV_DROPDOWN_PART_LIST, LV_STATE_DEFAULT, 255);
  lv_obj_set_style_local_radius(Dropdown1, LV_DROPDOWN_PART_LIST, LV_STATE_DEFAULT, 5);
  lv_obj_set_style_local_pad_left(Dropdown1, LV_DROPDOWN_PART_LIST, LV_STATE_DEFAULT, 5);
  lv_obj_set_style_local_pad_right(Dropdown1, LV_DROPDOWN_PART_LIST, LV_STATE_DEFAULT, 5);
  lv_obj_set_style_local_pad_top(Dropdown1, LV_DROPDOWN_PART_LIST, LV_STATE_DEFAULT, 0);
  lv_obj_set_style_local_pad_bottom(Dropdown1, LV_DROPDOWN_PART_LIST, LV_STATE_DEFAULT, 0);
  lv_obj_set_style_local_border_color(Dropdown1, LV_DROPDOWN_PART_SELECTED, LV_STATE_DEFAULT, lv_color_hex(181 * 256 * 256 + 218 * 256 + 255));
  lv_obj_set_style_local_border_opa(Dropdown1, LV_DROPDOWN_PART_SELECTED, LV_STATE_DEFAULT, 255);
  lv_obj_set_style_local_border_width(Dropdown1, LV_DROPDOWN_PART_SELECTED, LV_STATE_DEFAULT, 1);
  lv_obj_set_style_local_border_side(Dropdown1, LV_DROPDOWN_PART_SELECTED, LV_STATE_DEFAULT, LV_BORDER_SIDE_FULL);
  lv_obj_set_style_local_pad_left(Dropdown1, LV_DROPDOWN_PART_SELECTED, LV_STATE_DEFAULT, 5);
  lv_obj_set_style_local_pad_right(Dropdown1, LV_DROPDOWN_PART_SELECTED, LV_STATE_DEFAULT, 5);
  lv_obj_set_style_local_pad_top(Dropdown1, LV_DROPDOWN_PART_SELECTED, LV_STATE_DEFAULT, 0);
  lv_obj_set_style_local_pad_bottom(Dropdown1, LV_DROPDOWN_PART_SELECTED, LV_STATE_DEFAULT, 0);

  Dropdown2 = lv_dropdown_create(screen, Dropdown1);
  lv_dropdown_set_options( Dropdown2, "4\n6\n8\n10\n12\n14\n16\n18\n20\n22\n24");
  lv_obj_set_size(Dropdown2, 45, 20);
  lv_obj_align(Dropdown2, screen, LV_ALIGN_IN_TOP_LEFT, 185, 70);
  lv_dropdown_set_show_selected(Dropdown2, true);
  lv_obj_set_event_cb(Dropdown2, Dropdown2_eventhandler);

  Dropdown3 = lv_dropdown_create(screen, Dropdown1);
  lv_dropdown_set_options( Dropdown3, "en\nit\nfr\npt\nde\nes\nnl");
  lv_obj_set_size(Dropdown3, 45, 20);
  lv_obj_align(Dropdown3, screen, LV_ALIGN_IN_TOP_LEFT, 10, 13);
  lv_dropdown_set_show_selected(Dropdown3, true);
  lv_obj_set_event_cb(Dropdown3, Dropdown3_eventhandler);

  Sunrise1 = lv_label_create(screen, NULL);
  lv_label_set_long_mode(Sunrise1, LV_LABEL_LONG_CROP);
  lv_label_set_align(Sunrise1, LV_LABEL_ALIGN_CENTER);
  lv_label_set_text(Sunrise1, "Sunrise");
  lv_obj_set_size(Sunrise1, 135, 22);
  lv_obj_set_click(Sunrise1, false);
  lv_obj_set_hidden(Sunrise1, true);
  lv_obj_set_drag(Sunrise1, false);
  lv_obj_add_style(Sunrise1, LV_OBJ_PART_MAIN, &StyleBTN);
  lv_obj_set_style_local_text_font(Sunrise1, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, &lv_font_montserrat_16);
  lv_obj_align(Sunrise1, screen, LV_ALIGN_IN_TOP_LEFT, 45, 115);

  Sunrise2 = lv_label_create(screen, Sunrise1);
  lv_obj_align(Sunrise2, screen, LV_ALIGN_IN_TOP_LEFT, 45, 160);

  Lat1 = lv_label_create(screen, Sunrise1);
  lv_obj_align(Lat1, screen, LV_ALIGN_IN_TOP_LEFT, 45, 205);

  Lat2 = lv_label_create(screen, Sunrise1);
  lv_obj_align(Lat2, screen, LV_ALIGN_IN_TOP_LEFT, 45, 250);


  /* ---- END of GUI Objects ---- */

  LeggoPref( );
  AlertInterval =  millis();
}
/* ======      START LOOP       START LOOP      ======== */
void loop() {
  lv_task_handler();

  if ((WiFi.status() != WL_CONNECTED) ) {
    lv_obj_set_style_local_shadow_color(Led1, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, lv_color_hex(200 * 256 * 256 + 0 * 256 + 0 ));
    lv_obj_set_style_local_bg_color(Led1, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, lv_color_hex(200 * 256 * 256 + 0 * 256 + 0 ));
    lv_obj_set_style_local_shadow_width(Led1,  LV_BTN_PART_MAIN, LV_STATE_DEFAULT, 20);
    /*lv_refr_now(NULL);*/
    if ((millis() - previousMillis) >= interval) {
      Serial.println("Reconnecting to WiFi...");
      wifiConn();
      previousMillis = millis(); interval = 30000;
    }
  } else {
    ftpSrv.handleFTP();
  }
  if ((millis() - UpdateClock) >= 1000) {
    if (StartStatus == 3) {
      getLocalTime(&timeinfo) ;/*Serial.println(&timeinfo, "%A, %B %d %Y %H:%M:%S");//Tuesday, February 15 2022 09:58:05*/
      strftime(TT, sizeof(TT), "%H:%M:%S", &timeinfo);
      lv_label_set_text(Orologio1, TT);
      T = String(TT);
      CurrentHour = T.substring(0, 2);
      strftime(TT, sizeof(TT), "%d-%m-%Y", &timeinfo);
      DataOggi = String(TT); DataOggi.trim();
      if (AvvisoStart == 1) {
        if (lv_obj_is_visible(Avviso) == true) {
          lv_obj_set_hidden(Avviso, true);
        } else {
          lv_obj_set_hidden(Avviso, false);
        }
      }
      if (DataOggi != DataOld) {
        Serial.println("data oggi=" + DataOggi);
        Serial.println("data old=" + DataOld);
        T = "The date of the day has changed "; T += "\nNow I renew the table";
        TT[(T.length() + 1)];  T.toCharArray(TT, T.length() + 1);
        lv_label_set_text(Alert, TT);
        LeggoWeb();
        DataOld = DataOggi;
        T = DataOggi; TT[(T.length() + 1)];  T.toCharArray(TT, T.length() + 1);
        lv_label_set_text(Orologio2, TT);
        SalvoPref();
      }
    }
    UpdateClock = millis();
  }
  if ( (millis() - AlertInterval) > 5000) {
    lv_obj_set_hidden(Alert, true);
    if (StartStatus == 2) {
      StartStatus = 3;
      LeggoWeb();
    }
    AlertInterval =  millis();
  }

  delay(1);
}/* ----------- END LOOP     END LOOP     END LOOP  ------------*/

/* ==== PUT YOURS  ROUTINES HERE ====  lv_label_set_text(Alert, "");*/

void wifiConn() {
  WiFi.mode(WIFI_STA);
  WiFi.disconnect();
  delay(100);
  ssid.trim(); wifipw.trim();
  char ssid_char[40]; char wifipw_char[40];
  ssid.toCharArray(ssid_char, ssid.length() + 1); wifipw.toCharArray(wifipw_char, wifipw.length() + 1);
  WiFi.begin(ssid_char, wifipw_char);
  int maxRepeat = 0;
  while (WiFi.status() != WL_CONNECTED && maxRepeat < 20) {
    delay(1000);
    Serial.print("Connecting to WiFi..  "); Serial.println(ssid.c_str());
    maxRepeat++;
  }
  if (WiFi.status() == WL_CONNECTED) {
    StartStatus = 1;
    lv_label_set_text(Alert, " Connected to the WiFi network \n");
    T = ssid; T += "\nLocal IP = "; T += WiFi.localIP().toString().c_str();
    T += "\nNow I update the clock with NTP Client...wait";
    TT[(T.length() + 1)];  T.toCharArray(TT, T.length() + 1);
    lv_label_ins_text(Alert, LV_LABEL_POS_LAST, TT); lv_obj_align(Alert, screen, LV_ALIGN_IN_TOP_MID, 0, 13);
    lv_obj_set_hidden(Alert, false);
    lv_obj_move_foreground(Alert);
    Serial.print("Connected to the WiFi network ");    Serial.println(WiFi.localIP()); Serial.println(WiFi.macAddress());
    Serial.print(F("GW --> ")); Serial.println(WiFi.gatewayIP());
    Serial.print(F("SM --> "));  Serial.println(WiFi.subnetMask());
    Serial.print(F("DNS 1 --> ")); Serial.println(WiFi.dnsIP(0));
    Serial.print(F("DNS 2 --> "));  Serial.println(WiFi.dnsIP(1));
    lv_obj_set_style_local_shadow_color(Led1, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, lv_color_hex(100 * 256 * 256 + 255 * 256 + 100));
    lv_obj_set_style_local_bg_color(Led1, LV_BTN_PART_MAIN, LV_STATE_DEFAULT, lv_color_hex(100 * 256 * 256 + 255 * 256 + 100 ));
    lv_obj_set_style_local_shadow_width(Led1,  LV_BTN_PART_MAIN, LV_STATE_DEFAULT, 30);
    lv_led_set_bright(Led1, 255);
    lv_refr_now(NULL);
    ftpSrv.begin("Esp32", "Esp32");
    ReadLocalTime();
  } else {
    Serial.println("Timeout");
    lv_label_set_text(Alert, "TIMEOUT\nCheck if SSID and Password is correct in variables declaration\nAnd if WiFi is available...");
    lv_obj_align(Alert, screen, LV_ALIGN_IN_TOP_MID, 0, 13); lv_obj_move_foreground(Alert); lv_obj_set_hidden(Alert, false);
  }
  AlertInterval =  millis();
}

void SalvoPref( ) {/* SAVE PREFERENCES         SAVE PREFERENCES         SAVE PREFERENCES   */
  pFileOut = SPIFFS.open("/PTable.txt", "w"); /* 1. Open a stream for writing*/
  /*SPIFFS open function needs path and mode:  mode = "r", "w", "a", "r+", "w+", "a+" (text file) or "rb", "wb", "ab", "rb+", "wb+", "ab+" (binary)
    where r = read, w = write, a = append   + means file is open for update (read and write)   b means the file os open for binary operations*/
  if (pFileOut) {
    T = "\nPREFERENCES SAVED";
    TT[(T.length() + 1)];  T.toCharArray(TT, T.length() + 1);
    lv_label_ins_text(Alert, LV_LABEL_POS_LAST, TT); lv_obj_align(Alert, screen, LV_ALIGN_IN_TOP_MID, 0, 13);
    lv_obj_set_hidden(Alert, false); lv_obj_move_foreground(Alert);
    pFileOut.println(DataOggi); /* Date of day */
    pFileOut.println((String)CittaSelezionataPos); /* City selected in dropdown */
    pFileOut.println((String)MinWindPos); /* Min wind dropdown selected */
    pFileOut.println((String)LanguagePos); /* Min wind dropdown selected */
    pFileOut.close();
    Serial.print("write PREFERENCES");
  } else {
    Serial.print("cannot write PREFERENCES");
    T = "\nERROR Cannot write PREFERENCES";
    TT[(T.length() + 1)];  T.toCharArray(TT, T.length() + 1);
    lv_label_ins_text(Alert, LV_LABEL_POS_LAST, TT); lv_obj_align(Alert, screen, LV_ALIGN_IN_TOP_MID, 0, 13);
    lv_obj_set_hidden(Alert, false); lv_obj_move_foreground(Alert);
  }
}

void LeggoPref( ) {/* Read PREFERENCES       Read PREFERENCES       Read PREFERENCES       */
  pFileOut = SPIFFS.open("/PTable.txt", "r"); /* 1. Open a stream for writing*/
  /*SPIFFS open function needs path and mode:  mode = "r", "w", "a", "r+", "w+", "a+" (text file) or "rb", "wb", "ab", "rb+", "wb+", "ab+" (binary)
    where r = read, w = write, a = append   + means file is open for update (read and write)   b means the file os open for binary operations*/
  if (SPIFFS.exists("/PTable.txt")) {
    DataOld = pFileOut.readStringUntil('\n'); DataOld.trim();
    T = pFileOut.readStringUntil('\n'); CittaSelezionataPos = T.toInt();
    lv_dropdown_set_selected(Dropdown1, CittaSelezionataPos);
    lv_dropdown_get_selected_str(Dropdown1, TT, sizeof(TT));
    lv_obj_align(Dropdown1, screen, LV_ALIGN_IN_TOP_LEFT, 10, 70);
    Richiesta3 = Richiesta1 + String(TT) + Richiesta2;
    CittaSelezionata = String(TT);
    T = pFileOut.readStringUntil('\n'); MinWindPos = T.toInt();
    lv_dropdown_set_selected(Dropdown2, MinWindPos);
    lv_dropdown_get_selected_str(Dropdown2, TT, sizeof(TT));
    T = String(TT);
    MinWind = T.toInt();
    T = pFileOut.readStringUntil('\n'); LanguagePos = T.toInt();
    lv_dropdown_set_selected(Dropdown3, LanguagePos);
    lv_dropdown_get_selected_str(Dropdown3, TT, sizeof(TT));
    T = String(TT);    Language = T;
    pFileOut.close();
    PreferencesFound = 1;
  } else {
    Serial.print("cannot read PREFERENCES");
    T = "\nCannot read PREFERENCES\nI write default preferences now";
    TT[(T.length() + 1)];  T.toCharArray(TT, T.length() + 1);
    lv_label_ins_text(Alert, LV_LABEL_POS_LAST, TT); lv_obj_align(Alert, screen, LV_ALIGN_IN_TOP_MID, 0, 13);
    lv_obj_set_hidden(Alert, false); lv_obj_move_foreground(Alert);
    pFileOut.close();
  }
}
void LeggoWeb() {
  T = "Connection with Windfinder site"; T += "\nCity selected is " + CittaSelezionata;
  TT[(T.length() + 1)];  T.toCharArray(TT, T.length() + 1);
  lv_label_set_text(Alert,  TT);  lv_obj_align(Alert, screen, LV_ALIGN_IN_TOP_MID, 0, 13);
  lv_obj_set_hidden(Alert, false);  lv_obj_move_foreground(Alert);
  lv_refr_now(NULL);
  delay(100);
  HTTPClient http;
  String serverPath = Richiesta3;
  http.begin( serverPath.c_str());/* Your Domain name with URL path or IP address with path*/
  http.addHeader("Accept-Language", Language, false, true);/* ADD LANGUAGE TO HEADER */
  /* Send HTTP GET request*/
  int httpResponseCode = http.GET();

  if (httpResponseCode > 0) {
    /*Serial.print("HTTP Response code: "); Serial.println(httpResponseCode);*/
    if (httpResponseCode == 200) {
      T = "\nServer Response is OK";
      TT[(T.length() + 1)];  T.toCharArray(TT, T.length() + 1);
      lv_label_ins_text(Alert, LV_LABEL_POS_LAST, TT); lv_obj_align(Alert, screen, LV_ALIGN_IN_TOP_MID, 0, 13);
      /* get length of document (is -1 when Server sends no Content-Length header)*/
      int len = http.getSize();
      /* create buffer for read*/
      uint8_t buff[128] = { 0 };
      int StartRead = 0;
      int Trovo = 0;
      int TotLetto = 0;
      /* get tcp stream */
      WiFiClient * stream = http.getStreamPtr();
      pFileOut = SPIFFS.open("/test.txt", "w"); /* 1. Open a stream for writing*/
      /* read all data from server*/
      while (http.connected() && (len > 0 || len == -1)) {
        /* get available data size*/
        size_t size = stream->available();
        if (size) {
          /* read up to 128 byte*/
          int c = stream->readBytes(buff, ((size > sizeof(buff)) ? sizeof(buff) : size));
          String Received = String(buff, c); TotLetto += c;
          /*Serial.print("************"+ (String)TotLetto +"**********");
            Serial.print(Received);*/
          if (TotLetto > 12789) {/*CUT HEADER AND IMAGE*/
            StartRead = 1;
          }
          if (StartRead == 1)   {
            if (TotLetto > 61301) {
              goto bailout;
            }
            Received.replace(" ", "");/* CUT spaces */
            pFileOut.print(Received); /*Serial.print(Received);*/
          }
          /* write it to Serial*/
          /*Serial.write(buff, c);*/
          if (len > 0) {
            len -= c;
          }
        }
        delay(1);
      }
bailout:
      pFileOut.close(); /*  Close file */
      /*Serial.println();*/
      /*Serial.println("[HTTP] connection closed or file end");*/
    }  else {
      Serial.print("Error code: ");
      Serial.println(httpResponseCode);
      T = "\nServer Response is ERROR...Abort";
      TT[(T.length() + 1)];  T.toCharArray(TT, T.length() + 1);
      lv_label_ins_text(Alert, LV_LABEL_POS_LAST, TT); lv_obj_align(Alert, screen, LV_ALIGN_IN_TOP_MID, 0, 13);
      http.end();
      return;
    }
  }
  /* Free resources*/
  http.end();
  delay(200);
  lv_refr_now(NULL);
  Visualizzo();
}

void Visualizzo() {/* VISUALIZZO       VISUALIZZO      VISUALIZZO      VISUALIZZO      VISUALIZZO      VISUALIZZO      */
  String linea;
  int riga  = 0;
  int Rigabase = 0;
  int colon = 0;
  int i = 0;
  for (i = 0 ; i <= 4; i++) {
    Risult[i] = "";
  }
  AvvisoStart = 0; lv_obj_set_hidden(Avviso, true);
  InvioMail = 0 ; AggiungoTempo = 0;
  pFileOut = SPIFFS.open("/test.txt", "r");
  if (SPIFFS.exists("/test.txt")) {
    size_t len = pFileOut.size();
    for ( i = 0; i < len; i++) {
      linea = pFileOut.readStringUntil('\n');
      if (linea != "") {
        /*Serial.println((String)i + "  " + linea);*/
        Trovo1 = linea.indexOf("weathertable__header");/* DAY  */
        if (Trovo1 != -1) {
          linea = pFileOut.readStringUntil('\n');
          riga = 0 + Rigabase ; colon = 0; i++;
          T = linea; TT[(T.length() + 1)];  T.toCharArray(TT, T.length() + 1);
          lv_table_set_cell_value(Tavola1, riga, 0, TT);/*lv_table_set_cell_value(table, row, col, "Content")*/
          /*Serial.println("*********************** Giorno=" + linea + "   riga=" + (String)riga + "    Col=" + (String)colon);*/
        }
        Trovo1 = linea.indexOf("<spanclass=\"value\">");/* HOUR  */
        if (Trovo1 != -1) {
          colon ++;
        }
        Trovo1 = linea.indexOf("<spanclass=\"units-ws\">");/* WIND WIND WIND */
        if (Trovo1 != -1) {
          if (Trovo1 == 0) {/* WIND base */
            Trovo2 = linea.indexOf(">") ; Trovo3 = linea.indexOf("<", Trovo2);
            T = linea.substring((Trovo2 + 1), Trovo3);
            riga = 1 + Rigabase;
            TT[(T.length() + 1)];  T.toCharArray(TT, T.length() + 1);
            lv_table_set_cell_value(Tavola1, riga, colon, TT);/*lv_table_set_cell_value(table, row, col, "Content")*/
            AggiungoTempo = 0;
            /* Serial.println("Risult 2=" + Risult[2] + "    CurrHour " + (String)CurrentHour.toInt() + "    CurrHour String " + CurrentHour);*/
            if ( T.toInt() >= MinWind) { /* Risult[4] is     0 City       1 Day        2 Hour       3 Wind min        4 Cloud cover */
              /* dont consider hour 7 and hour 19, dont consider if  hour is lower  that actual hour */
              Trovo3 =  (String( lv_table_get_cell_value(Tavola1, 0, colon))).toInt();
              if (InvioMail == 0  && colon > 1 && colon < 5 ) {
                if (Rigabase == 0) {/* if is current day we consider current hour */
                  if (CurrentHour.toInt() <= Trovo3) {
                    Risult[0] = CittaSelezionata;
                    Risult[1] = lv_table_get_cell_value(Tavola1, Rigabase, 0);
                    Risult[2] =  lv_table_get_cell_value(Tavola1, 0, colon);
                    Risult[3] = T;
                    InvioMail = 1 ; AggiungoTempo = 1;
                  }
                } else {
                  //  Serial.println("T=" + T + "    Trovo3=" + (String)Trovo3 + "    CurrHour " + (String)CurrentHour.toInt() + "    Colon= " + (String)colon);
                  Risult[0] = CittaSelezionata;
                  Risult[1] = lv_table_get_cell_value(Tavola1, Rigabase, 0);
                  Risult[2] =  lv_table_get_cell_value(Tavola1, 0, colon);
                  Risult[3] = T;
                  InvioMail = 1 ; AggiungoTempo = 1;
                }
              }
            }
          }
          if (Trovo1 == 39) {/* WIND MAX  */
            Trovo2 = linea.indexOf(">", (Trovo1 + 1)) ; Trovo3 = linea.indexOf("<", Trovo2);
            T = linea.substring((Trovo2 + 1), Trovo3);
            riga = 2 + Rigabase;
            TT[(T.length() + 1)];  T.toCharArray(TT, T.length() + 1);
            lv_table_set_cell_value(Tavola1, riga, colon, TT);/*lv_table_set_cell_value(table, row, col, "Content")*/
          }
        }
        Trovo1 = linea.indexOf("data-cover__symbolicon-");/* CLOUD COVER */
        if (Trovo1 != -1) {
          String Tempo2;
          Trovo1 = linea.indexOf("-skcunits");/* SUN */
          if (Trovo1 != -1) Tempo2 = "0";
          Trovo1 = linea.indexOf("-sct");/* Sun mix */
          if (Trovo1 != -1) Tempo2 = "25";
          Trovo1 = linea.indexOf("-few");/* Light Cover*/
          if (Trovo1 != -1) Tempo2 = "50";
          Trovo1 = linea.indexOf("-bkn");/* Cover */
          if (Trovo1 != -1) Tempo2 = "75";
          Trovo1 = linea.indexOf("-ovc");/* Big Cover */
          if (Trovo1 != -1) Tempo2 = "100";
          riga = 3 + Rigabase;
          T = Tempo2; TT[(T.length() + 1)];  T.toCharArray(TT, T.length() + 1);
          /*Serial.println("*********************** Tempo=" + linea + "   riga=" + (String)riga + "    Col=" + (String)colon);*/
          lv_table_set_cell_value(Tavola1, riga, colon, TT);
          if (AggiungoTempo == 1) {
            Risult[4] = Tempo2 ;
            if (Risult[4].toInt() == 100) {
              AggiungoTempo = 0; InvioMail = 0; /*Warning cancelled because is all cover by clouds */
            }
          }
          if ( colon == 5) {
            Rigabase += 4 ; colon = 0;
          }
        }
        Trovo1 = linea.indexOf("<divclass=\"widget-spotmeta\">");/* Sunrise */
        if (Trovo1 != -1) {
          for ( int r = 1; r <= 5; r++) {
            linea = pFileOut.readStringUntil('\n');
          }
          T = "Sunrise " + linea; TT[(T.length() + 1)];  T.toCharArray(TT, T.length() + 1);
          lv_label_set_text(Sunrise1, TT);
          for ( int r = 1; r <= 7; r++) {
            linea = pFileOut.readStringUntil('\n');
          }
          T = "Sunset " + linea; TT[(T.length() + 1)];  T.toCharArray(TT, T.length() + 1);
          lv_label_set_text(Sunrise2, TT);
        }

        Trovo1 = linea.indexOf("Lat:</strong>");/* Lat / Long */
        if (Trovo1 != -1) {
          Trovo2 = linea.indexOf(">", (Trovo1 + 1)); Trovo3 = linea.indexOf("<", (Trovo2 + 1));
          T = "Lat: " + linea.substring((Trovo2 + 1), Trovo3);
          TT[(T.length() + 1)];  T.toCharArray(TT, T.length() + 1);
          lv_label_set_text(Lat1, TT);
          linea = pFileOut.readStringUntil('\n');
          Trovo1 = linea.indexOf("Lon:</strong>");
          Trovo2 = linea.indexOf(">", (Trovo1 + 1)); Trovo3 = linea.indexOf("<", (Trovo2 + 1));
          T = "Lon: " + linea.substring((Trovo2 + 1), Trovo3);  T.toCharArray(TT, T.length() + 1);
          lv_label_set_text(Lat2, TT);
        }
      }
    }
  }
  pFileOut.close(); /*  Close file */
  T = "\nTable Data Refreshed"; TT[(T.length() + 1)];  T.toCharArray(TT, T.length() + 1);
  lv_label_ins_text(Alert, LV_LABEL_POS_LAST, TT);  lv_obj_align(Alert, screen, LV_ALIGN_IN_TOP_MID, 0, 13);
  if (InvioMail == 1) {
    /*Risult[4] As String ' 0 City     1 Day       2 Hour       3 min wind speed       4 cloud cover */
    T = "\n" + Risult[0] + "   " + Risult[1] + "   Hour " + Risult[2] + "   Wind Kmh " + Risult[3] + "  Clouds " + Risult[4];
    if (Risult[4].toInt() == 100) {
      T += "\nWarning canceled because there is bad weather" ;
      TT[(T.length() + 1)];  T.toCharArray(TT, T.length() + 1);
      lv_label_ins_text(Alert, LV_LABEL_POS_LAST, TT);  lv_obj_align(Alert, screen, LV_ALIGN_IN_TOP_MID, 0, 13);
      InvioMail = 0;
    } else {
      /*            If (you want send Email == True) {   */
      /*          build a soubroutine     SendEmail()  */
      /*            }  */
      String R = Risult[0].substring(0, 16) + "  " + Risult[1] + "   Hour " + Risult[2] + "   Wind " + Risult[3] + "  Clouds " + Risult[4];
      TT[(R.length() + 1)];  R.toCharArray(TT, R.length() + 1);
      lv_label_set_text(Avviso,  TT); lv_obj_align(Avviso, screen,  LV_ALIGN_CENTER, 0, 110); lv_obj_move_foreground(Avviso);
      T += "\nI found some wind" ;
      TT[(T.length() + 1)];  T.toCharArray(TT, T.length() + 1);
      lv_label_ins_text(Alert, LV_LABEL_POS_LAST, TT);  lv_obj_align(Alert, screen, LV_ALIGN_IN_TOP_MID, 0, 13);
      AvvisoStart = 1;
    }
  }
  lv_obj_set_hidden(Alert, false);  lv_obj_move_foreground(Alert);
  lv_obj_set_hidden(Dropdown1, false); lv_obj_set_hidden(Dropdown2, false); lv_obj_set_hidden(Dropdown3, false);
  lv_obj_set_hidden(Sunrise1, false); lv_obj_set_hidden(Sunrise2, false);
  lv_obj_set_hidden(Lat1, false); lv_obj_set_hidden(Lat2, false);
  AlertInterval =  millis();
  lv_refr_now(NULL);
}

void ReadLocalTime() {
  int RetryNTP = 0;
  String Anno = "";
  configTime(0, 0, "it.pool.ntp.org", "europe.pool.ntp.org", "time.nist.gov");
  T = TimeZone; TT[(T.length() + 1)];  T.toCharArray(TT, T.length() + 1);
  setenv("TZ", TT, 1);/* Rome https://github.com/nayarsystems/posix_tz_db/blob/master/zones.csv */
  tzset();
  getLocalTime(&timeinfo) ;
  strftime(TT, sizeof(TT), "%Y", &timeinfo);
  Anno = String(TT);
  Serial.println("Year  = " + Anno);

  while (Anno == "1970" && RetryNTP < 10) {
    RetryNTP ++;
    getLocalTime(&timeinfo);
    strftime(TT, sizeof(TT), "%Y", &timeinfo);
    Anno = String(TT);
    Serial.println("Year ***** " + Anno);
    T = "."; TT[(T.length() + 1)];  T.toCharArray(TT, T.length() + 1);
    lv_label_ins_text(Alert, LV_LABEL_POS_LAST, TT); lv_obj_align(Alert, screen, LV_ALIGN_IN_TOP_MID, 0, 13);
  }

  now = time(nullptr);
  T = ctime(&now);
  Serial.println("Time now  = " + T);
  if (Anno != "1970") {
    StartStatus = 2;
    Serial.println(&timeinfo, "%A, %B %d %Y %H:%M:%S");/*Tuesday, February 15 2022 09:58:05 */
    strftime(TT, sizeof(TT), "%H:", &timeinfo);
    T = String(TT);
    CurrentHour = T.substring(0, 2);
    strftime(TT, sizeof(TT), "%d-%m-%Y", &timeinfo);/* here you can build format date...if change them , change them also in main loop routine */
    DataOggi = String(TT);
    T = "\nToday Date is " + DataOggi ;
    TT[(T.length() + 1)];  T.toCharArray(TT, T.length() + 1);
    lv_label_ins_text(Alert, LV_LABEL_POS_LAST,  TT); lv_obj_align(Alert, screen, LV_ALIGN_IN_TOP_MID, 0, 13);
    T = DataOggi; TT[(T.length() + 1)];  T.toCharArray(TT, T.length() + 1);
    lv_label_set_text(Orologio2, TT);
    lv_obj_set_hidden(Orologio1, false); lv_obj_set_hidden(Orologio2, false);
    if (PreferencesFound == 0) {
      SalvoPref();
      PreferencesFound = 1;
      delay (500);
      LeggoPref();
      delay (500);
    }
  } else {
    T += "Failed to get current Date";
    TT[(T.length() + 1)];  T.toCharArray(TT, T.length() + 1);
    lv_label_set_text(Alert, TT); lv_obj_align(Alert, screen, LV_ALIGN_IN_TOP_MID, 0, 13);
  }
}
/* APPENDIX
  %a  Abbreviated weekday name.
  %A  Full weekday name.
  %b  Abbreviated month name.
  %B  Full month name.
  %c  Date/Time in the format of the locale.
  %C  Century number [00-99], the year divided by 100 and truncated to an integer.
  %d  Day of the month [01-31].
  %D  Date Format, same as %m/%d/%y.
  %e  Same as %d, except single digit is preceded by a space [1-31].
  %g  2 digit year portion of ISO week date [00,99].
  %F  ISO Date Format, same as %Y-%m-%d.
  %G  4 digit year portion of ISO week date. Can be negative.
  %h  Same as %b.
  %H  Hour in 24-hour format [00-23].
  %I  Hour in 12-hour format [01-12].
  %j  Day of the year [001-366].
  %m  Month [01-12].
  %M  Minute [00-59].
  %n  Newline character.
  %p  AM or PM string.
  %r  Time in AM/PM format of the locale. If not available in the locale time format, defaults to the POSIX time AM/PM format: %I:%M:%S %p.
  %R  24-hour time format without seconds, same as %H:%M.
  %S  Second [00-61]. The range for seconds allows for a leap second and a double leap second.
  %t  Tab character.
  %T  24-hour time format with seconds, same as %H:%M:%S.
  %u  Weekday [1,7]. Monday is 1 and Sunday is 7.
  %U  Week number of the year [00-53]. Sunday is the first day of the week.
  %V  ISO week number of the year [01-53]. Monday is the first day of the week. If the week containing January 1st has four or more days in the new year then it is considered week 1. Otherwise, it is the last week of the previous year, and the next year is week 1 of the new year.
  %w  Weekday [0,6], Sunday is 0.
  %W  Week number of the year [00-53]. Monday is the first day of the week.
  %x  Date in the format of the locale.
  %X  Time in the format of the locale.
  %y  2 digit year [00,99].
  %Y  4-digit year. Can be negative.
  %z  UTC offset. Output is a string with format +HHMM or -HHMM, where + indicates east of GMT, - indicates west of GMT, HH indicates the number of hours from GMT, and MM indicates the number of minutes from GMT.
  %Z  Time zone name.
*/
